use anyhow::{Context, Result};
use minijinja::{context, Environment};
use quick_xml::escape::escape;
use reqwest::Client;
use std::fs::{File, OpenOptions};
use std::io::Write;
use std::path::{Path, PathBuf};
use std::time::Duration;
use tokio::time::timeout;

const TEMPLATE_STRING: &str = "
<item>
    <title> {{ title }} </title>
    <link> {{ link }} </link>
</item>

";

pub struct LinkFile {
    file: File,
    client: Client,
    path: PathBuf,
}

async fn get_link_title(client: &Client, link: &str) -> Result<String> {
    let res = timeout(
        Duration::from_secs(120),
        timeout(Duration::from_secs(120), client.get(link).send())
            .await??
            .text(),
    )
    .await??;

    let dom = tl::parse(&res, tl::ParserOptions::default())?;
    let elem = dom
        .query_selector("title")
        .and_then(|mut iter| iter.next())
        .context("No title")?;
    let node = elem.get(dom.parser()).context("No title")?;
    Ok(node.inner_text(dom.parser()).to_string())
}

impl LinkFile {
    pub fn new(path: impl AsRef<Path> + Clone) -> Self {
        let file = OpenOptions::new().append(true).open(path.clone()).unwrap();
        let client = Client::new();
        Self {
            file,
            client,
            path: path.as_ref().to_path_buf(),
        }
    }

    pub async fn add_link(&mut self, link: &str) -> Result<()> {
        let title = match get_link_title(&self.client, link).await {
            Ok(s) => s,
            Err(_) => escape(link).to_string(),
        };

        let mut env = Environment::new();
        env.add_template("rss_fragment", TEMPLATE_STRING)?;
        let template = env.get_template("rss_fragment")?;

        self.file.write_all(
            template
                .render(context! {
                title => title,
                link => escape(link).to_string(),
                })?
                .as_bytes(),
        )?;
        Ok(())
    }

    pub fn get_path(&self) -> PathBuf {
        self.path.clone()
    }
}

#[cfg(test)]
mod tests {

    use crate::writer::LinkFile;

    #[test]
    fn test_linkfile() {
        let mut linkfile = LinkFile::new("./store.xmlfrag");
        linkfile.add_link("https://evilmuff.in").unwrap();
    }
}
